@@ -33,6 +33,7 @@ typedef struct BoolCompressor
 	Simple8bRleCompressor validity_bitmap;
 	bool has_nulls;
 	bool last_value;
+	uint32 num_nulls;
 } BoolCompressor;
 
 typedef struct ExtendedCompressor
@@ -88,6 +89,7 @@ bool_compressor_append_null(BoolCompressor *compressor)
 	compressor->has_nulls = true;
 	simple8brle_compressor_append(&compressor->values, compressor->last_value);
 	simple8brle_compressor_append(&compressor->validity_bitmap, 0);
+	compressor->num_nulls++;
 }
 
 extern void
@@ -108,6 +110,9 @@ bool_compressor_finish(BoolCompressor *compressor)
 	if (values == NULL)
 		return NULL;
 
+	if (compressor->num_nulls == compressor->values.num_elements)
+		return NULL;
+
 	Simple8bRleSerialized *validity_bitmap =
 		simple8brle_compressor_finish(&compressor->validity_bitmap);
 	BoolCompressed *compressed;
