@@ -34,6 +34,10 @@
 #include <nodes/modify_hypertable.h>
 #include <ts_catalog/array_utils.h>
 
+typedef BatchQualSummary(BatchMatcher)(RowDecompressor *decompressor, ScanKeyData *scankeys,
+									   int num_scankeys, tuple_filtering_constraints *constraints,
+									   bool check_full_match, bool *skip_current_tuple);
+
 static struct decompress_batches_stats
 decompress_batches_scan(Relation in_rel, Relation out_rel, Relation index_rel, Snapshot snapshot,
 						ScanKeyData *index_scankeys, int num_index_scankeys,
@@ -42,11 +46,13 @@ decompress_batches_scan(Relation in_rel, Relation out_rel, Relation index_rel, S
 						tuple_filtering_constraints *constraints, bool *skip_current_tuple,
 						bool delete_only, Bitmapset *null_columns, List *is_nulls);
 
-static bool batch_matches(RowDecompressor *decompressor, ScanKeyData *scankeys, int num_scankeys,
-						  tuple_filtering_constraints *constraints, bool *skip_current_tuple);
-static bool batch_matches_vectorized(RowDecompressor *decompressor, ScanKeyData *scankeys,
-									 int num_scankeys, tuple_filtering_constraints *constraints,
-									 bool *skip_current_tuple);
+static BatchQualSummary batch_matches(RowDecompressor *decompressor, ScanKeyData *scankeys,
+									  int num_scankeys, tuple_filtering_constraints *constraints,
+									  bool check_full_match, bool *skip_current_tuple);
+static BatchQualSummary batch_matches_vectorized(RowDecompressor *decompressor,
+												 ScanKeyData *scankeys, int num_scankeys,
+												 tuple_filtering_constraints *constraints,
+												 bool check_full_match, bool *skip_current_tuple);
 static void process_predicates(Chunk *ch, CompressionSettings *settings, List *predicates,
 							   ScanKeyData **mem_scankeys, int *num_mem_scankeys,
 							   List **heap_filters, List **index_filters, List **is_null);
@@ -57,8 +63,6 @@ get_batch_keys_for_unique_constraints(const ChunkInsertState *cis, Relation rela
 static BatchFilter *make_batchfilter(char *column_name, StrategyNumber strategy, Oid collation,
 									 RegProcedure opcode, Const *value, bool is_null_check,
 									 bool is_null, bool is_array_op);
-static inline TM_Result delete_compressed_tuple(RowDecompressor *decompressor, Snapshot snapshot,
-												HeapTuple compressed_tuple);
 static void report_error(TM_Result result);
 
 static bool key_column_is_null(tuple_filtering_constraints *constraints, Relation chunk_rel,
@@ -68,8 +72,9 @@ static bool can_delete_without_decompression(ModifyHypertableState *ht_state,
 											 List *predicates);
 static bool can_vectorize_constraint_checks(tuple_filtering_constraints *constraints,
 											CompressionSettings *settings, Relation chunk_rel,
-											Oid ht_relid, ScanKeyData *mem_scankeys,
-											int num_mem_scankeys);
+											Oid ht_relid, ScanKeyWithAttnos *mem_scankeys);
+static ScanKeyData *get_updated_scankeys(const ScanKeyWithAttnos *scankeys, TupleTableSlot *slot,
+										 int null_flags);
 
 static AttrNumber
 TupleDescGetAttrNumber(TupleDesc desc, const char *name)
@@ -79,11 +84,141 @@ TupleDescGetAttrNumber(TupleDesc desc, const char *name)
 		if (strcmp(name, NameStr(desc->attrs[i].attname)) == 0)
 			return desc->attrs[i].attnum;
 	}
+
 	return InvalidAttrNumber;
 }
 
 void
-decompress_batches_for_insert(const ChunkInsertState *cis, TupleTableSlot *slot)
+init_decompress_state_for_insert(ChunkInsertState *cis, TupleTableSlot *slot)
+{
+	if (!cis->chunk_compressed || cis->cached_decompression_state != NULL)
+	{
+		/*
+		 * If the chunk is not compressed or the decompression state has
+		 * already been initialized, there is nothing to do here.
+		 */
+		return;
+	}
+
+	CachedDecompressionState *cdst = NULL;
+
+	MemoryContext old_context = MemoryContextSwitchTo(cis->mctx);
+	cdst = palloc0(sizeof(CachedDecompressionState));
+
+	cdst->has_primary_or_unique_index = ts_indexing_relation_has_primary_or_unique_index(cis->rel);
+
+	if (cdst->has_primary_or_unique_index)
+	{
+		tuple_filtering_constraints *constraints =
+			get_batch_keys_for_unique_constraints(cis, cis->rel);
+		cdst->constraints = constraints;
+
+		CompressionSettings *compression_settings =
+			ts_compression_settings_get(RelationGetRelid(cis->rel));
+		Assert(compression_settings && OidIsValid(compression_settings->fd.compress_relid));
+		cdst->compression_settings = compression_settings;
+
+		Relation in_rel = relation_open(compression_settings->fd.compress_relid, RowExclusiveLock);
+
+		Bitmapset *columns_with_null_check = NULL;
+		Bitmapset *key_columns = constraints->key_columns;
+		Bitmapset *index_columns = NULL;
+		Relation index_rel = NULL;
+
+		if (ts_guc_enable_dml_decompression_tuple_filtering)
+		{
+			cdst->mem_scankeys.scankeys =
+				build_mem_scankeys_from_slot(cis->hypertable_relid,
+											 compression_settings,
+											 cis->rel,
+											 constraints,
+											 slot,
+											 &cdst->mem_scankeys.num_scankeys,
+											 &cdst->mem_scankeys.attnos);
+
+			cdst->constraints->vectorized_filtering =
+				can_vectorize_constraint_checks(constraints,
+												compression_settings,
+												cis->rel,
+												cis->hypertable_relid,
+												&cdst->mem_scankeys);
+
+			cdst->index_scankeys.scankeys =
+				build_index_scankeys_using_slot(cis->hypertable_relid,
+												in_rel,
+												cis->rel,
+												constraints->key_columns,
+												slot,
+												&index_rel,
+												&index_columns,
+												&cdst->index_scankeys.num_scankeys,
+												&cdst->index_scankeys.attnos);
+		}
+
+		if (index_rel)
+		{
+			/*
+			 * Prepare the heap scan keys for all
+			 * key columns not found in the index
+			 */
+			key_columns = bms_difference(constraints->key_columns, index_columns);
+		}
+
+		cdst->heap_scankeys.scankeys = build_heap_scankeys(cis->hypertable_relid,
+														   in_rel,
+														   cis->rel,
+														   compression_settings,
+														   key_columns,
+														   &columns_with_null_check,
+														   slot,
+														   &cdst->heap_scankeys.num_scankeys,
+														   &cdst->heap_scankeys.attnos);
+
+		if (index_rel)
+		{
+			cdst->index_relid = RelationGetRelid(index_rel);
+			columns_with_null_check = NULL;
+			index_close(index_rel, AccessShareLock);
+		}
+
+		cdst->columns_with_null_check = columns_with_null_check;
+		table_close(in_rel, NoLock);
+	}
+	cis->cached_decompression_state = cdst;
+
+	MemoryContextSwitchTo(old_context);
+}
+
+static ScanKeyData *
+get_updated_scankeys(const ScanKeyWithAttnos *scankeys, TupleTableSlot *slot, int null_flags)
+{
+	if (scankeys->num_scankeys == 0)
+	{
+		return NULL;
+	}
+
+	ScanKeyData *updated_scankeys = palloc0(sizeof(ScanKeyData) * scankeys->num_scankeys);
+	for (int i = 0; i < scankeys->num_scankeys; i++)
+	{
+		updated_scankeys[i] = scankeys->scankeys[i];
+		bool isnull = false;
+		Datum value = slot_getattr(slot, scankeys->attnos[i], &isnull);
+		if (isnull)
+		{
+			updated_scankeys[i].sk_flags = null_flags;
+			updated_scankeys[i].sk_argument = (Datum) 0;
+		}
+		else
+		{
+			updated_scankeys[i].sk_flags = 0;
+			updated_scankeys[i].sk_argument = value;
+		}
+	}
+	return updated_scankeys;
+}
+
+void
+decompress_batches_for_insert(ChunkInsertState *cis, TupleTableSlot *slot)
 {
 	/*
 	 * This is supposed to be called with the actual tuple that is being
@@ -92,8 +227,10 @@ decompress_batches_for_insert(const ChunkInsertState *cis, TupleTableSlot *slot)
 	Assert(!TTS_EMPTY(slot));
 
 	Relation out_rel = cis->rel;
+	CachedDecompressionState *cdst = cis->cached_decompression_state;
+	Assert(cdst != NULL);
 
-	if (!ts_indexing_relation_has_primary_or_unique_index(out_rel))
+	if (!cdst->has_primary_or_unique_index)
 	{
 		/*
 		 * If there are no unique constraints there is nothing to do here.
@@ -107,8 +244,7 @@ decompress_batches_for_insert(const ChunkInsertState *cis, TupleTableSlot *slot)
 				 errmsg("inserting into compressed chunk with unique constraints disabled"),
 				 errhint("Set timescaledb.enable_dml_decompression to TRUE.")));
 
-	tuple_filtering_constraints *constraints = get_batch_keys_for_unique_constraints(cis, out_rel);
-	if (key_column_is_null(constraints, out_rel, cis->hypertable_relid, slot))
+	if (key_column_is_null(cdst->constraints, cis->rel, cis->hypertable_relid, slot))
 	{
 		/* When any key column is NULL and NULLs are distinct there is no
 		 * decompression to be done as the tuple will not conflict with any
@@ -117,80 +253,34 @@ decompress_batches_for_insert(const ChunkInsertState *cis, TupleTableSlot *slot)
 		return;
 	}
 
-	CompressionSettings *settings = ts_compression_settings_get(RelationGetRelid(cis->rel));
-	Assert(settings && OidIsValid(settings->fd.compress_relid));
-	Relation in_rel = relation_open(settings->fd.compress_relid, RowExclusiveLock);
-	Bitmapset *index_columns = NULL;
-	Bitmapset *null_columns = NULL;
-	struct decompress_batches_stats stats;
+	Assert(cdst->compression_settings->fd.relid == RelationGetRelid(out_rel));
+	Relation in_rel =
+		relation_open(cdst->compression_settings->fd.compress_relid, RowExclusiveLock);
 
 	/* the scan keys used for in memory tests of the decompressed tuples */
-	int num_mem_scankeys = 0;
-	ScanKeyData *mem_scankeys = NULL;
-	int num_index_scankeys = 0;
-	ScanKeyData *index_scankeys = NULL;
-	Relation index_rel = NULL;
-	ScanKeyData *heap_scankeys = NULL;
-	int num_heap_scankeys = 0;
-	Bitmapset *key_columns = constraints->key_columns;
-
-	if (ts_guc_enable_dml_decompression_tuple_filtering)
-	{
-		mem_scankeys = build_mem_scankeys_from_slot(cis->hypertable_relid,
-													settings,
-													out_rel,
-													constraints,
-													slot,
-													&num_mem_scankeys);
-
-		constraints->vectorized_filtering = can_vectorize_constraint_checks(constraints,
-																			settings,
-																			out_rel,
-																			cis->hypertable_relid,
-																			mem_scankeys,
-																			num_mem_scankeys);
-
-		index_scankeys = build_index_scankeys_using_slot(cis->hypertable_relid,
-														 in_rel,
-														 out_rel,
-														 constraints->key_columns,
-														 slot,
-														 &index_rel,
-														 &index_columns,
-														 &num_index_scankeys);
-	}
-
 	bool skip_current_tuple = false;
-	if (index_rel)
+	struct decompress_batches_stats stats;
+
+	Relation index_rel = NULL;
+	if (OidIsValid(cdst->index_relid))
 	{
-		/*
-		 * Prepare the heap scan keys for all
-		 * key columns not found in the index
-		 */
-		key_columns = bms_difference(constraints->key_columns, index_columns);
+		index_rel = index_open(cdst->index_relid, AccessShareLock);
 	}
 
-	heap_scankeys = build_heap_scankeys(cis->hypertable_relid,
-										in_rel,
-										out_rel,
-										settings,
-										key_columns,
-										&null_columns,
-										slot,
-										&num_heap_scankeys);
-
-	/* no null column check for non-segmentby columns in case of index scan */
-	if (index_rel)
-		null_columns = NULL;
+	ScanKeyData *index_scankeys =
+		get_updated_scankeys(&cdst->index_scankeys, slot, SK_ISNULL | SK_SEARCHNULL);
+	ScanKeyData *heap_scankeys =
+		get_updated_scankeys(&cdst->heap_scankeys, slot, SK_ISNULL | SK_SEARCHNULL);
+	ScanKeyData *mem_scankeys = get_updated_scankeys(&cdst->mem_scankeys, slot, SK_ISNULL);
 
 	if (ts_guc_debug_compression_path_info)
 	{
 		elog(INFO,
 			 "Using %s scan with scan keys: index %d, heap %d, memory %d. ",
-			 index_rel ? "index" : "table",
-			 num_index_scankeys,
-			 num_heap_scankeys,
-			 num_mem_scankeys);
+			 OidIsValid(cdst->index_relid) ? "index" : "table",
+			 cdst->index_scankeys.num_scankeys,
+			 cdst->heap_scankeys.num_scankeys,
+			 cdst->mem_scankeys.num_scankeys);
 	}
 
 	/*
@@ -203,30 +293,36 @@ decompress_batches_for_insert(const ChunkInsertState *cis, TupleTableSlot *slot)
 									index_rel,
 									GetLatestSnapshot(),
 									index_scankeys,
-									num_index_scankeys,
+									cdst->index_scankeys.num_scankeys,
 									heap_scankeys,
-									num_heap_scankeys,
+									cdst->heap_scankeys.num_scankeys,
 									mem_scankeys,
-									num_mem_scankeys,
-									constraints,
+									cdst->mem_scankeys.num_scankeys,
+									cdst->constraints,
 									&skip_current_tuple,
 									false,
-									null_columns, /* no null column check for non-segmentby
-											 columns */
+									cdst->columns_with_null_check, /* no null column check for
+														   non-segmentby columns */
 									NIL);
 	if (index_rel)
 		index_close(index_rel, AccessShareLock);
 
-	Assert(cis->cds != NULL);
 	if (skip_current_tuple)
 	{
-		cis->cds->skip_current_tuple = true;
+		cis->skip_current_tuple = true;
 	}
 
-	cis->cds->batches_deleted += stats.batches_deleted;
-	cis->cds->batches_filtered += stats.batches_filtered;
-	cis->cds->batches_decompressed += stats.batches_decompressed;
-	cis->cds->tuples_decompressed += stats.tuples_decompressed;
+	cis->counters->batches_deleted += stats.batches_deleted;
+	cis->counters->batches_filtered += stats.batches_filtered;
+	cis->counters->batches_decompressed += stats.batches_decompressed;
+	cis->counters->tuples_decompressed += stats.tuples_decompressed;
+
+	if (index_scankeys)
+		pfree(index_scankeys);
+	if (heap_scankeys)
+		pfree(heap_scankeys);
+	if (mem_scankeys)
+		pfree(mem_scankeys);
 
 	CommandCounterIncrement();
 	table_close(in_rel, NoLock);
@@ -429,6 +525,7 @@ decompress_batches_scan(Relation in_rel, Relation out_rel, Relation index_rel, S
 						bool delete_only, Bitmapset *null_columns, List *is_nulls)
 {
 	HeapTuple compressed_tuple;
+	BulkWriter writer;
 	RowDecompressor decompressor;
 	bool decompressor_initialized = false;
 	bool valid = false;
@@ -523,9 +620,9 @@ decompress_batches_scan(Relation in_rel, Relation out_rel, Relation index_rel, S
 
 		if (!decompressor_initialized)
 		{
-			decompressor = build_decompressor(in_rel, out_rel);
-			decompressor.delete_only = delete_only;
+			decompressor = build_decompressor(RelationGetDescr(in_rel), RelationGetDescr(out_rel));
 			decompressor_initialized = true;
+			writer = bulk_writer_build(out_rel, 0);
 			meta_count_attno = TupleDescGetAttrNumber(decompressor.in_desc,
 													  COMPRESSION_COLUMN_METADATA_COUNT_NAME);
 			Assert(meta_count_attno != InvalidAttrNumber);
@@ -536,15 +633,24 @@ decompress_batches_scan(Relation in_rel, Relation out_rel, Relation index_rel, S
 						  decompressor.compressed_datums,
 						  decompressor.compressed_is_nulls);
 
-		if (num_mem_scankeys && !batch_matcher(&decompressor,
-											   mem_scankeys,
-											   num_mem_scankeys,
-											   constraints,
-											   skip_current_tuple))
+		/* If there are no in-memory quals, all rows pass */
+		BatchQualSummary summary = AllRowsPass;
+		if (num_mem_scankeys)
 		{
-			row_decompressor_reset(&decompressor);
-			stats.batches_filtered++;
-			continue;
+			summary = batch_matcher(&decompressor,
+									mem_scankeys,
+									num_mem_scankeys,
+									constraints,
+									delete_only, /* need to check full batch for direct DELETEs */
+									skip_current_tuple);
+
+			/* If no rows pass, complete batch gets filtered */
+			if (summary == NoRowsPass)
+			{
+				row_decompressor_reset(&decompressor);
+				stats.batches_filtered++;
+				continue;
+			}
 		}
 
 		row_decompressor_reset(&decompressor);
@@ -552,12 +658,23 @@ decompress_batches_scan(Relation in_rel, Relation out_rel, Relation index_rel, S
 		if (skip_current_tuple && *skip_current_tuple)
 		{
 			row_decompressor_close(&decompressor);
+			bulk_writer_close(&writer);
 			decompress_batch_endscan(scan);
 			ExecDropSingleTupleTableSlot(slot);
 			return stats;
 		}
 		write_logical_replication_msg_decompression_start();
-		result = delete_compressed_tuple(&decompressor, snapshot, compressed_tuple);
+
+		TM_FailureData tmfd;
+		result = table_tuple_delete(in_rel,
+									&compressed_tuple->t_self,
+									GetCurrentCommandId(true),
+									snapshot,
+									InvalidSnapshot,
+									true,
+									&tmfd,
+									false);
+
 		/* skip reporting error if isolation level is < Repeatable Read
 		 * since somebody decompressed the data concurrently, we need to take
 		 * that data into account as well when in Read Committed level
@@ -572,11 +689,13 @@ decompress_batches_scan(Relation in_rel, Relation out_rel, Relation index_rel, S
 		{
 			write_logical_replication_msg_decompression_end();
 			row_decompressor_close(&decompressor);
+			bulk_writer_close(&writer);
 			decompress_batch_endscan(scan);
 			report_error(result);
 			return stats;
 		}
-		if (decompressor.delete_only)
+		/* If all rows pass, complete batch can be deleted */
+		if (delete_only && summary == AllRowsPass)
 		{
 			stats.batches_deleted++;
 			stats.tuples_deleted += DatumGetInt32(
@@ -584,7 +703,8 @@ decompress_batches_scan(Relation in_rel, Relation out_rel, Relation index_rel, S
 		}
 		else
 		{
-			stats.tuples_decompressed += row_decompressor_decompress_row_to_table(&decompressor);
+			stats.tuples_decompressed +=
+				row_decompressor_decompress_row_to_table(&decompressor, &writer);
 			stats.batches_decompressed++;
 		}
 		write_logical_replication_msg_decompression_end();
@@ -594,6 +714,7 @@ decompress_batches_scan(Relation in_rel, Relation out_rel, Relation index_rel, S
 	if (decompressor_initialized)
 	{
 		row_decompressor_close(&decompressor);
+		bulk_writer_close(&writer);
 	}
 
 	if (ts_guc_debug_compression_path_info)
@@ -610,9 +731,10 @@ decompress_batches_scan(Relation in_rel, Relation out_rel, Relation index_rel, S
 	return stats;
 }
 
-static bool
+static BatchQualSummary
 batch_matches(RowDecompressor *decompressor, ScanKeyData *scankeys, int num_scankeys,
-			  tuple_filtering_constraints *constraints, bool *skip_current_tuple)
+			  tuple_filtering_constraints *constraints, bool check_full_match,
+			  bool *skip_current_tuple)
 {
 	AttrNumber *attnos = palloc0(sizeof(AttrNumber) * num_scankeys);
 	for (int i = 0; i < num_scankeys; i++)
@@ -624,6 +746,12 @@ batch_matches(RowDecompressor *decompressor, ScanKeyData *scankeys, int num_scan
 	ScanKey key;
 	bool match;
 
+	/* Default values are set like this because of binary operations
+	 * used to calculate these flags.
+	 */
+	bool match_any = false;
+	bool match_all = true;
+
 	while (next_tuple)
 	{
 		match = true;
@@ -658,8 +786,12 @@ batch_matches(RowDecompressor *decompressor, ScanKeyData *scankeys, int num_scan
 			}
 		}
 
+		match_any |= match;
+		match_all &= match;
+
 		if (match)
 		{
+			match_any = true;
 			if (constraints)
 			{
 				if (constraints->on_conflict == ONCONFLICT_NONE)
@@ -676,13 +808,19 @@ batch_matches(RowDecompressor *decompressor, ScanKeyData *scankeys, int num_scan
 					*skip_current_tuple = true;
 				}
 			}
-			return true;
+			if (!check_full_match)
+				return SomeRowsPass;
 		}
-
 		next_tuple = decompress_batch_next_row(decompressor, attnos, num_scankeys);
 	}
 
-	return false;
+	if (match_all)
+		return AllRowsPass;
+
+	if (match_any)
+		return SomeRowsPass;
+
+	return NoRowsPass;
 }
 
 static void
@@ -713,9 +851,10 @@ check_single_value_match(const uint64 *result)
 	return result[0] & 1;
 }
 
-static bool
+static BatchQualSummary
 batch_matches_vectorized(RowDecompressor *decompressor, ScanKeyData *scankeys, int num_scankeys,
-						 tuple_filtering_constraints *constraints, bool *skip_current_tuple)
+						 tuple_filtering_constraints *constraints, bool check_full_match,
+						 bool *skip_current_tuple)
 {
 	const int n_rows =
 		DatumGetInt32(decompressor->compressed_datums[decompressor->count_compressed_attindex]);
@@ -773,10 +912,10 @@ batch_matches_vectorized(RowDecompressor *decompressor, ScanKeyData *scankeys, i
 
 	if (batch_failed)
 	{
-		return false;
+		return NoRowsPass;
 	}
 
-	VectorQualSummary summary = get_vector_qual_summary(result, n_rows);
+	BatchQualSummary summary = get_vector_qual_summary(result, n_rows);
 
 	if (summary != NoRowsPass)
 	{
@@ -796,10 +935,10 @@ batch_matches_vectorized(RowDecompressor *decompressor, ScanKeyData *scankeys, i
 				*skip_current_tuple = true;
 			}
 		}
-		return true;
+		return summary;
 	}
 
-	return false;
+	return summary;
 }
 
 /*
@@ -920,6 +1059,9 @@ decompress_chunk_walker(PlanState *ps, struct decompress_chunk_context *ctx)
 				 * data. To circumvent this issue, we change the internal scan state to use the
 				 * transaction snapshot and execute a rescan so the scan state is set correctly and
 				 * includes the new data.
+				 *
+				 * From PG17 this has changed since the scan state is not initialized with
+				 * the node.
 				 */
 				if (should_rescan)
 				{
@@ -927,7 +1069,7 @@ decompress_chunk_walker(PlanState *ps, struct decompress_chunk_context *ctx)
 					if (ss && ss->ss_currentScanDesc)
 					{
 						ss->ss_currentScanDesc->rs_snapshot = GetTransactionSnapshot();
-						ExecReScan(ps);
+						table_rescan(ss->ss_currentScanDesc, NULL);
 					}
 				}
 			}
@@ -1034,7 +1176,7 @@ get_batch_keys_for_unique_constraints(const ChunkInsertState *cis, Relation rela
 			return constraints;
 	}
 
-	if (constraints->covered && cis->cds->dispatch)
+	if (constraints->covered && cis->cds && cis->cds->dispatch)
 	{
 		constraints->on_conflict = ts_chunk_dispatch_get_on_conflict_action(cis->cds->dispatch);
 	}
@@ -1166,89 +1308,93 @@ process_predicates(Chunk *ch, CompressionSettings *settings, List *predicates,
 																 var->varattno,
 																 settings->fd.compress_relid,
 																 "min");
+				if (min_attno == InvalidAttrNumber)
+					continue;
+
 				int max_attno = compressed_column_metadata_attno(settings,
 																 ch->table_id,
 																 var->varattno,
 																 settings->fd.compress_relid,
 																 "max");
+				if (max_attno == InvalidAttrNumber)
+					continue;
 
-				if (min_attno != InvalidAttrNumber && max_attno != InvalidAttrNumber)
+				/* Need both min and max metadata attributes to build heap filters */
+
+				switch (op_strategy)
 				{
-					switch (op_strategy)
+					case BTEqualStrategyNumber:
 					{
-						case BTEqualStrategyNumber:
-						{
-							/* orderby col = value implies min <= value and max >= value */
-							*heap_filters =
-								lappend(*heap_filters,
-										make_batchfilter(get_attname(settings->fd.compress_relid,
-																	 min_attno,
-																	 false),
-														 BTLessEqualStrategyNumber,
-														 collation,
-														 opcode,
-														 arg_value,
-														 false, /* is_null_check */
-														 false, /* is_null */
-														 false	/* is_array_op */
-														 ));
-							*heap_filters =
-								lappend(*heap_filters,
-										make_batchfilter(get_attname(settings->fd.compress_relid,
-																	 max_attno,
-																	 false),
-														 BTGreaterEqualStrategyNumber,
-														 collation,
-														 opcode,
-														 arg_value,
-														 false, /* is_null_check */
-														 false, /* is_null */
-														 false	/* is_array_op */
-														 ));
-						}
-						break;
-						case BTLessStrategyNumber:
-						case BTLessEqualStrategyNumber:
-						{
-							/* orderby col <[=] value implies min <[=] value */
-							*heap_filters =
-								lappend(*heap_filters,
-										make_batchfilter(get_attname(settings->fd.compress_relid,
-																	 min_attno,
-																	 false),
-														 op_strategy,
-														 collation,
-														 opcode,
-														 arg_value,
-														 false, /* is_null_check */
-														 false, /* is_null */
-														 false	/* is_array_op */
-														 ));
-						}
-						break;
-						case BTGreaterStrategyNumber:
-						case BTGreaterEqualStrategyNumber:
-						{
-							/* orderby col >[=] value implies max >[=] value */
-							*heap_filters =
-								lappend(*heap_filters,
-										make_batchfilter(get_attname(settings->fd.compress_relid,
-																	 max_attno,
-																	 false),
-														 op_strategy,
-														 collation,
-														 opcode,
-														 arg_value,
-														 false, /* is_null_check */
-														 false, /* is_null */
-														 false	/* is_array_op */
-														 ));
-						}
-						break;
-						default:
-							/* Do nothing for unknown operator strategies. */
-							break;
+						/* orderby col = value implies min <= value and max >= value */
+						*heap_filters =
+							lappend(*heap_filters,
+									make_batchfilter(get_attname(settings->fd.compress_relid,
+																 min_attno,
+																 false),
+													 BTLessEqualStrategyNumber,
+													 collation,
+													 opcode,
+													 arg_value,
+													 false, /* is_null_check */
+													 false, /* is_null */
+													 false	/* is_array_op */
+													 ));
+						*heap_filters =
+							lappend(*heap_filters,
+									make_batchfilter(get_attname(settings->fd.compress_relid,
+																 max_attno,
+																 false),
+													 BTGreaterEqualStrategyNumber,
+													 collation,
+													 opcode,
+													 arg_value,
+													 false, /* is_null_check */
+													 false, /* is_null */
+													 false	/* is_array_op */
+													 ));
 					}
+					break;
+					case BTLessStrategyNumber:
+					case BTLessEqualStrategyNumber:
+					{
+						/* orderby col <[=] value implies min <[=] value */
+						*heap_filters =
+							lappend(*heap_filters,
+									make_batchfilter(get_attname(settings->fd.compress_relid,
+																 min_attno,
+																 false),
+													 op_strategy,
+													 collation,
+													 opcode,
+													 arg_value,
+													 false, /* is_null_check */
+													 false, /* is_null */
+													 false	/* is_array_op */
+													 ));
+					}
+					break;
+					case BTGreaterStrategyNumber:
+					case BTGreaterEqualStrategyNumber:
+					{
+						/* orderby col >[=] value implies max >[=] value */
+						*heap_filters =
+							lappend(*heap_filters,
+									make_batchfilter(get_attname(settings->fd.compress_relid,
+																 max_attno,
+																 false),
+													 op_strategy,
+													 collation,
+													 opcode,
+													 arg_value,
+													 false, /* is_null_check */
+													 false, /* is_null */
+													 false	/* is_array_op */
+													 ));
+					}
+					break;
+					default:
+						/* Do nothing for unknown operator strategies. */
+						break;
 				}
 			}
 			break;
@@ -1501,23 +1647,6 @@ find_matching_index(Relation comp_chunk_rel, List **index_filters, List **heap_f
 	return result_rel;
 }
 
-static inline TM_Result
-delete_compressed_tuple(RowDecompressor *decompressor, Snapshot snapshot,
-						HeapTuple compressed_tuple)
-{
-	TM_FailureData tmfd;
-	TM_Result result;
-	result = table_tuple_delete(decompressor->in_rel,
-								&compressed_tuple->t_self,
-								decompressor->mycid,
-								snapshot,
-								InvalidSnapshot,
-								true,
-								&tmfd,
-								false);
-	return result;
-}
-
 static void
 report_error(TM_Result result)
 {
@@ -1632,9 +1761,23 @@ can_delete_without_decompression(ModifyHypertableState *ht_state, CompressionSet
 				return false;
 			}
 			char *column_name = get_attname(chunk->table_id, var->varattno, false);
+			/* Can do direct DELETE if we are dealing with segmentby columns */
 			if (ts_array_is_member(settings->fd.segmentby, column_name))
-			{
 				continue;
+
+			/* Can do direct DELETE if we are using in-memory filtering but
+			 * only if we can actually create scankeys for filtering
+			 */
+			if (ts_guc_enable_dml_decompression_tuple_filtering)
+			{
+				switch (nodeTag(node))
+				{
+					case T_ScalarArrayOpExpr:
+					case T_NullTest:
+						return false;
+					default:
+						continue;
+				}
 			}
 		}
 		return false;
@@ -1645,19 +1788,29 @@ can_delete_without_decompression(ModifyHypertableState *ht_state, CompressionSet
 static bool
 can_vectorize_constraint_checks(tuple_filtering_constraints *constraints,
 								CompressionSettings *settings, Relation chunk_rel, Oid ht_relid,
-								ScanKeyData *mem_scankeys, int num_mem_scankeys)
+								ScanKeyWithAttnos *mem_scankeys)
 {
 	AttrNumber chunk_attno = -1;
 	Oid typoid, collid;
 	int32 typmod;
 
-	if (mem_scankeys == NULL || num_mem_scankeys == 0)
+	if (mem_scankeys == NULL || mem_scankeys->num_scankeys == 0)
 		return false;
 
 	/* We can only vectorize if a vectorized check is available for all scankeys */
-	for (int sk = 0; sk < num_mem_scankeys; sk++)
+	for (int sk = 0; sk < mem_scankeys->num_scankeys; sk++)
 	{
-		ScanKeyData *scankey = &mem_scankeys[sk];
+		/*
+		 * Here we cannot check for NULL flags even if that is
+		 * handled separately, because this code is called from
+		 * the `init_decompress_state_for_insert` which sets the
+		 * flag based on the first record to be inserted and the
+		 * value may change for the subsequent records.
+		 *
+		 * The `fn_oid` doesn't get updated so it is valid to check
+		 * it here.
+		 */
+		ScanKeyData *scankey = &mem_scankeys->scankeys[sk];
 		if (get_vector_const_predicate(scankey->sk_func.fn_oid) == NULL)
 			return false;
 	}
