@@ -481,7 +481,9 @@ caggtimebucket_validate(CAggTimebucketInfo *tbinfo, List *groupClause, List *tar
 	}
 
 	if (!found)
-		elog(ERROR, "continuous aggregate view must include a valid time bucket function");
+		ereport(ERROR,
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+				 errmsg("continuous aggregate view must include a valid time bucket function")));
 }
 
 /*
@@ -517,9 +519,20 @@ cagg_query_supported(const Query *query, StringInfo hint, StringInfo detail, con
 
 	if (query->hasWindowFuncs)
 	{
-		appendStringInfoString(detail,
-							   "Window functions are not supported by continuous aggregates.");
-		return false;
+		if (ts_guc_enable_cagg_window_functions)
+		{
+			elog(WARNING,
+				 "window function support is experimental and may result in unexpected results "
+				 "depending on the functions used.");
+		}
+		else
+		{
+			appendStringInfoString(detail, "Window function support not enabled.");
+			appendStringInfoString(hint,
+								   "Enable experimental window function support by setting "
+								   "timescaledb.enable_cagg_window_functions.");
+			return false;
+		}
 	}
 
 	if (query->hasDistinctOn || query->distinctClause)
@@ -758,7 +771,7 @@ cagg_validate_query(const Query *query, const bool finalized, const char *cagg_s
 
 		if (!ht)
 		{
-			ts_cache_release(hcache);
+			ts_cache_release(&hcache);
 			ereport(ERROR,
 					(errcode(ERRCODE_TS_HYPERTABLE_NOT_EXIST),
 					 errmsg("table \"%s\" is not a hypertable", get_rel_name(rte->relid))));
@@ -770,7 +783,7 @@ cagg_validate_query(const Query *query, const bool finalized, const char *cagg_s
 
 		if (!cagg_parent)
 		{
-			ts_cache_release(hcache);
+			ts_cache_release(&hcache);
 			ereport(ERROR,
 					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 					 errmsg("invalid continuous aggregate query"),
@@ -780,7 +793,7 @@ cagg_validate_query(const Query *query, const bool finalized, const char *cagg_s
 
 		if (!ContinuousAggIsFinalized(cagg_parent))
 		{
-			ts_cache_release(hcache);
+			ts_cache_release(&hcache);
 			ereport(ERROR,
 					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 					 errmsg("old format of continuous aggregate is not supported"),
@@ -808,7 +821,7 @@ cagg_validate_query(const Query *query, const bool finalized, const char *cagg_s
 
 	if (TS_HYPERTABLE_IS_INTERNAL_COMPRESSION_TABLE(ht))
 	{
-		ts_cache_release(hcache);
+		ts_cache_release(&hcache);
 		ereport(ERROR,
 				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 				 errmsg("hypertable is an internal compressed hypertable")));
@@ -825,7 +838,7 @@ cagg_validate_query(const Query *query, const bool finalized, const char *cagg_s
 				ts_continuous_agg_find_by_mat_hypertable_id(ht->fd.id, false);
 			Assert(cagg != NULL);
 
-			ts_cache_release(hcache);
+			ts_cache_release(&hcache);
 			ereport(ERROR,
 					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 					 errmsg("hypertable is a continuous aggregate materialization table"),
@@ -848,7 +861,7 @@ cagg_validate_query(const Query *query, const bool finalized, const char *cagg_s
 	 */
 	if (part_dimension == NULL || part_dimension->partitioning != NULL)
 	{
-		ts_cache_release(hcache);
+		ts_cache_release(&hcache);
 		ereport(ERROR,
 				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 				 errmsg("custom partitioning functions not supported"
@@ -863,7 +876,7 @@ cagg_validate_query(const Query *query, const bool finalized, const char *cagg_s
 
 		if (strlen(funcschema) == 0 || strlen(funcname) == 0)
 		{
-			ts_cache_release(hcache);
+			ts_cache_release(&hcache);
 			ereport(ERROR,
 					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 					 errmsg("custom time function required on hypertable \"%s\"",
@@ -895,7 +908,7 @@ cagg_validate_query(const Query *query, const bool finalized, const char *cagg_s
 								INVALID_HYPERTABLE_ID);
 	}
 
-	ts_cache_release(hcache);
+	ts_cache_release(&hcache);
 
 	/*
 	 * We need a GROUP By clause with time_bucket on the partitioning
@@ -1293,7 +1306,7 @@ makeRangeTblEntry(Query *query, const char *aliasname)
 
 	rte->lateral = false;
 	rte->inh = false; /* never true for subqueries */
-	rte->inFromCl = true;
+	rte->inFromCl = false;
 
 	return rte;
 }
@@ -1425,11 +1438,11 @@ build_union_query(CAggTimebucketInfo *tbinfo, int matpartcolno, Query *q1, Query
 	}
 
 	query->targetList = tlist;
+	query->jointree = makeFromExpr(NIL, NULL);
 
 	if (sortClause)
 	{
 		query->sortClause = sortClause;
-		query->jointree = makeFromExpr(NIL, NULL);
 	}
 
 	setop->colTypes = col_types;
