@@ -41,14 +41,13 @@ extern void invalidation_hyper_log_add_entry(int32 hyper_id, int64 start, int64
 extern void continuous_agg_invalidate_raw_ht(const Hypertable *raw_ht, int64 start, int64 end);
 extern void continuous_agg_invalidate_mat_ht(const Hypertable *raw_ht, const Hypertable *mat_ht,
 											 int64 start, int64 end);
-
-extern void invalidation_process_hypertable_log(const ContinuousAgg *cagg, Oid dimtype,
-												const CaggsInfo *all_caggs_info);
+extern Datum continuous_agg_process_hypertable_invalidations(PG_FUNCTION_ARGS);
+extern void invalidation_process_hypertable_log(int32 hypertable_id, Oid dimtype);
 
 extern InvalidationStore *
 invalidation_process_cagg_log(const ContinuousAgg *cagg, const InternalTimeRange *refresh_window,
-							  const CaggsInfo *all_caggs_info, const long max_materializations,
-							  bool *do_merged_refresh, InternalTimeRange *ret_merged_refresh_window,
-							  const CaggRefreshContext context, bool force);
+							  long max_materializations, bool *do_merged_refresh,
+							  InternalTimeRange *ret_merged_refresh_window,
+							  CaggRefreshContext context, bool force);
 
 extern void invalidation_store_free(InvalidationStore *store);
