@@ -1147,6 +1147,7 @@ continuous_agg_split_refresh_window(ContinuousAgg *cagg, InternalTimeRange *orig
 			WHERE \
 				hypertable_id = $1 \
 				AND dimension_id = $2 \
+				AND range_end >= range_start \
 			ORDER BY \
 				%s \
 		), \
@@ -1158,6 +1159,7 @@ continuous_agg_split_refresh_window(ContinuousAgg *cagg, InternalTimeRange *orig
 				_timescaledb_catalog.continuous_aggs_materialization_invalidation_log \
 			WHERE \
 				materialization_id = $3 \
+				AND greatest_modified_value >= lowest_modified_value \
 			UNION ALL \
 			SELECT \
 				pg_catalog.min(lowest_modified_value) AS lowest_modified_value, \
@@ -1166,6 +1168,7 @@ continuous_agg_split_refresh_window(ContinuousAgg *cagg, InternalTimeRange *orig
 				_timescaledb_catalog.continuous_aggs_hypertable_invalidation_log \
 			WHERE \
 				hypertable_id = $1 \
+				AND greatest_modified_value >= lowest_modified_value \
 		) \
 		SELECT \
 			refresh_start AS start, \
@@ -1290,7 +1293,7 @@ continuous_agg_split_refresh_window(ContinuousAgg *cagg, InternalTimeRange *orig
 			 (batch == 0 && !refresh_newest_first)) &&
 			original_refresh_window->start_isnull)
 		{
-			range->start = ts_time_get_nobegin_or_min(range->type);
+			range->start = cagg_get_time_min(cagg);
 			range->start_isnull = true;
 		}
 
