@@ -16,11 +16,11 @@ extern void continuous_agg_calculate_merged_refresh_window(
 	const ContinuousAgg *cagg, const InternalTimeRange *refresh_window,
 	const InvalidationStore *invalidations, const ContinuousAggsBucketFunction *bucket_function,
 	InternalTimeRange *merged_refresh_window, const CaggRefreshContext context);
-extern void continuous_agg_refresh_internal(const ContinuousAgg *cagg,
-											const InternalTimeRange *refresh_window,
-											const CaggRefreshContext context,
-											const bool start_isnull, const bool end_isnull,
-											bool force);
+extern void
+continuous_agg_refresh_internal(const ContinuousAgg *cagg, const InternalTimeRange *refresh_window,
+								const CaggRefreshContext context, const bool start_isnull,
+								const bool end_isnull, bool bucketing_refresh_window, bool force,
+								bool process_hypertable_invalidations, bool extend_last_bucket);
 extern List *continuous_agg_split_refresh_window(ContinuousAgg *cagg,
 												 InternalTimeRange *original_refresh_window,
 												 int32 buckets_per_batch,
