@@ -381,7 +381,9 @@ decompress_chunk_begin(CustomScanState *node, EState *estate, int eflags)
 
 	if (ts_guc_debug_require_batch_sorted_merge && !dcontext->batch_sorted_merge)
 	{
-		elog(ERROR, "debug: batch sorted merge is required but not used");
+		ereport(ERROR,
+				(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
+				 errmsg("debug: batch sorted merge is required but not used")));
 	}
 
 	/* Constify stable expressions in vectorized predicates. */
@@ -522,7 +524,12 @@ decompress_chunk_explain(CustomScanState *node, List *ancestors, ExplainState *e
 			ExplainPropertyBool("Batch Sorted Merge", dcontext->batch_sorted_merge, es);
 		}
 
-		if (es->analyze && (es->verbose || es->format != EXPLAIN_FORMAT_TEXT))
+		if (dcontext->reverse)
+		{
+			ExplainPropertyBool("Reverse", dcontext->reverse, es);
+		}
+
+		if (es->analyze)
 		{
 			ExplainPropertyBool("Bulk Decompression",
 								chunk_state->decompress_context.enable_bulk_decompression,
