@@ -44,6 +44,14 @@ static CustomScanMethods decompress_chunk_plan_methods = {
 	.CreateCustomScanState = decompress_chunk_state_create,
 };
 
+/* Check if the provided plan is a DecompressChunkPlan */
+bool
+ts_is_decompress_chunk_plan(Plan *plan)
+{
+	return IsA(plan, CustomScan) &&
+		   castNode(CustomScan, plan)->methods == &decompress_chunk_plan_methods;
+}
+
 void
 _decompress_chunk_init(void)
 {
@@ -272,7 +280,7 @@ build_decompression_map(DecompressionMapContext *context, List *compressed_outpu
 	}
 #endif
 	/*
-	 * FIXME this way to determine which columns are used is actually wrong, see
+	 * TODO this way to determine which columns are used is actually wrong, see
 	 * https://github.com/timescale/timescaledb/issues/4195#issuecomment-1104238863
 	 * Left as is for now, because changing it uncovers a whole new story with
 	 * ctid.
@@ -653,6 +661,8 @@ is_not_runtime_constant_walker(Node *node, void *context)
  * Note that we do the same evaluation when doing run time chunk exclusion, but
  * there is no good way to pass the evaluated clauses to the underlying nodes
  * like this DecompressChunk node.
+ *
+ * Similar checks are performed for sparse index pushdown.
  */
 static bool
 is_not_runtime_constant(Node *node)
@@ -679,9 +689,23 @@ vector_qual_make(Node *qual, const VectorQualInfo *vqinfo)
 		{
 			/*
 			 * NOT should be removed by Postgres for all operators we can
-			 * vectorize (see prepqual.c), so we don't support it.
+			 * vectorize (see prepqual.c) except when the where clause is
+			 * something like 'COL = false' for bool columns. In this case, we
+			 * have to check if it was transformed to BoolExpr(NOT_EXPR, Var) so
+			 * we can vectorize it, provided that the column supports bulk
+			 * decompression.
 			 */
-			return NULL;
+			if (list_length(boolexpr->args) == 1 && IsA(linitial(boolexpr->args), Var))
+			{
+				if (!vqinfo->vector_attrs[castNode(Var, linitial(boolexpr->args))->varattno])
+				{
+					return NULL;
+				}
+			}
+			else
+			{
+				return NULL;
+			}
 		}
 
 		bool need_copy = false;
@@ -716,7 +740,8 @@ vector_qual_make(Node *qual, const VectorQualInfo *vqinfo)
 
 	/*
 	 * Among the simple predicates, we vectorize some "Var op Const" binary
-	 * predicates, scalar array operations with these predicates, and null test.
+	 * predicates, scalar array operations with these predicates, boolean variables
+	 * and null test.
 	 */
 	NullTest *nulltest = NULL;
 	OpExpr *opexpr = NULL;
@@ -724,6 +749,7 @@ vector_qual_make(Node *qual, const VectorQualInfo *vqinfo)
 	Node *arg1 = NULL;
 	Node *arg2 = NULL;
 	Oid opno = InvalidOid;
+	Var *var = NULL;
 	if (IsA(qual, OpExpr))
 	{
 		opexpr = castNode(OpExpr, qual);
@@ -748,6 +774,33 @@ vector_qual_make(Node *qual, const VectorQualInfo *vqinfo)
 		nulltest = castNode(NullTest, qual);
 		arg1 = (Node *) nulltest->arg;
 	}
+	else if (IsA(qual, BooleanTest))
+	{
+		BooleanTest *booltest = castNode(BooleanTest, qual);
+		if (IsA(booltest->arg, Var))
+		{
+			var = castNode(Var, booltest->arg);
+			if (!vqinfo->vector_attrs[var->varattno])
+			{
+				return NULL;
+			}
+			return (Node *) booltest;
+		}
+		else
+		{
+			return NULL;
+		}
+	}
+	else if (IsA(qual, Var) && (castNode(Var, qual))->vartype == BOOLOID)
+	{
+		/* We can vectorize boolean variables if bulk decompression is possible. */
+		var = castNode(Var, qual);
+		if (!vqinfo->vector_attrs[var->varattno])
+		{
+			return NULL;
+		}
+		return (Node *) var;
+	}
 	else
 	{
 		return NULL;
@@ -785,7 +838,7 @@ vector_qual_make(Node *qual, const VectorQualInfo *vqinfo)
 		return NULL;
 	}
 
-	Var *var = castNode(Var, arg1);
+	var = castNode(Var, arg1);
 	if ((Index) var->varno != vqinfo->rti)
 	{
 		/*
@@ -965,9 +1018,10 @@ ts_label_sort_with_costsize(PlannerInfo *root, Sort *plan, double limit_tuples)
 static Var *
 find_var_subexpression(void *expr, Index varno)
 {
-	if (IsA(expr, Var))
+	List *varlist = pull_var_clause((Node *) expr, 0);
+	if (list_length(varlist) == 1)
 	{
-		Var *var = castNode(Var, expr);
+		Var *var = (Var *) linitial(varlist);
 		if ((Index) var->varno == (Index) varno)
 		{
 			return var;
@@ -976,27 +1030,6 @@ find_var_subexpression(void *expr, Index varno)
 		return NULL;
 	}
 
-	if (IsA(expr, List))
-	{
-		List *list = castNode(List, expr);
-		ListCell *lc;
-		foreach (lc, list)
-		{
-			Var *var = find_var_subexpression(lfirst(lc), varno);
-			if (var != NULL)
-			{
-				return var;
-			}
-		}
-
-		return NULL;
-	}
-
-	if (IsA(expr, FuncExpr))
-	{
-		return find_var_subexpression(castNode(FuncExpr, expr)->args, varno);
-	}
-
 	return NULL;
 }
 
@@ -1343,17 +1376,23 @@ decompress_chunk_plan_create(PlannerInfo *root, RelOptInfo *rel, CustomPath *pat
 #ifdef TS_DEBUG
 	if (ts_guc_debug_require_vector_qual == DRO_Forbid && list_length(vectorized_quals) > 0)
 	{
-		elog(ERROR, "debug: encountered vector quals when they are disabled");
+		ereport(ERROR,
+				(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
+				 errmsg("debug: encountered vector quals when they are disabled")));
 	}
 	else if (ts_guc_debug_require_vector_qual == DRO_Require)
 	{
 		if (list_length(decompress_plan->scan.plan.qual) > 0)
 		{
-			elog(ERROR, "debug: encountered non-vector quals when they are disabled");
+			ereport(ERROR,
+					(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
+					 errmsg("debug: encountered non-vector quals when they are disabled")));
 		}
 		if (list_length(vectorized_quals) == 0)
 		{
-			elog(ERROR, "debug: did not encounter vector quals when they are required");
+			ereport(ERROR,
+					(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
+					 errmsg("debug: did not encounter vector quals when they are required")));
 		}
 	}
 #endif
