@@ -537,10 +537,12 @@ INSERT INTO test_ordering VALUES (5),(4),(3);
 -- should be ordered append
 :PREFIX SELECT * FROM test_ordering ORDER BY 1;
 SELECT compress_chunk(format('%I.%I',chunk_schema,chunk_name), true) FROM timescaledb_information.chunks WHERE hypertable_name = 'test_ordering';
+VACUUM ANALYZE test_ordering;
 
 -- should be ordered append
 :PREFIX SELECT * FROM test_ordering ORDER BY 1;
 INSERT INTO test_ordering SELECT 1;
+VACUUM ANALYZE test_ordering;
 
 -- should not be ordered append
 -- regression introduced by #5599:
@@ -552,6 +554,8 @@ INSERT INTO test_ordering SELECT 1;
 :PREFIX SELECT * FROM test_ordering ORDER BY 1;
 
 INSERT INTO test_ordering VALUES (105),(104),(103);
+VACUUM ANALYZE test_ordering;
+
 -- should be ordered append
 :PREFIX SELECT * FROM test_ordering ORDER BY 1;
 
@@ -564,6 +568,7 @@ INSERT INTO test_ordering VALUES (23), (24), (115) RETURNING *;
 INSERT INTO test_ordering VALUES (23), (24), (115) RETURNING tableoid::regclass, *;
 
 SELECT compress_chunk(format('%I.%I',chunk_schema,chunk_name), true) FROM timescaledb_information.chunks WHERE hypertable_name = 'test_ordering';
+VACUUM ANALYZE test_ordering;
 
 -- should be ordered append
 :PREFIX SELECT * FROM test_ordering ORDER BY 1;
@@ -898,6 +903,7 @@ INSERT INTO unique_null VALUES
 ('2024-01-01 00:00', 1, NULL, '2', 2),
 ('2024-01-01 00:00', 1, NULL, '3', 3);
 SELECT count(compress_chunk(c)) FROM show_chunks('unique_null') c;
+
 \set ON_ERROR_STOP 0
 INSERT INTO unique_null VALUES ('2024-01-01 00:00', 1, NULL, '1', 1);
 \set ON_ERROR_STOP 1
@@ -910,6 +916,10 @@ CREATE UNIQUE INDEX unique_nulls ON unique_null(time, device_id, tag, tag2, tag3
 INSERT INTO unique_null VALUES ('2024-01-01 00:00', 1, NULL, '1', 1, 'default value');
 \set ON_ERROR_STOP 1
 
+INSERT INTO unique_null VALUES
+	('2024-01-01 00:01', 1, NULL, '1', 1, 'default value'),
+	('2024-01-01 00:00', 2, NULL, '1', 1, 'default value');
+
 DROP TABLE unique_null;
 
 -- test NUMERIC data type
